ELECTRUM_VERSION = '2.6.4'  # version of the client package
PROTOCOL_VERSION = '0.10'   # protocol version requested
NEW_SEED_VERSION = 11       # electrum versions >= 2.0
OLD_SEED_VERSION = 4        # electrum versions < 2.0


# The hash of the mnemonic seed must begin with this
SEED_PREFIX      = '01'      # Electrum standard wallet
SEED_PREFIX_2FA  = '101'     # extended seed for two-factor authentication
