'''
OSC
===

This is an heavy modified version of PyOSC that Kivy uses internally for TUIO
providers.
'''

__version__ = "0"
__author__ = "www.ixi-software.net"
__license__ = "GNU General Public License"
__all__ = ("oscAPI", "OSC")

from .OSC import *
from .oscAPI import *


