'''
External libraries
==================

Kivy comes with other python/C libraries :

* ddslib
* oscAPI (modified / optimized)
* mtdev

.. warning::

    Even though Kivy comes with these external libraries, we do not provide any
    support for them and they might change in the future.
    Don't rely on them in your code.

'''
